<?php

include (dirname(__FILE__).'/../Libs/kpf/flowAPI.php');

class Tuxpan_FlowPayment_CheckoutController extends Mage_Core_Controller_Front_Action
{
	public function gatewayAction()
	{
		$orderId = Mage::getSingleton('checkout/session')->getLastRealOrderId();
		$order = Mage::getSingleton('sales/order')->loadByIncrementId($orderId);

		$orden_compra = $orderId;
		$monto = $order->getGrandTotal();
		$concepto = 'Orden #'.$orderId.' - '.Mage::app()->getStore()->getName();
		$email_pagador = $order->getCustomerEmail();

		$flowAPI = $this->getFlowAPI();
		try {
			$flow_pack = $flowAPI->new_order($orden_compra, $monto, $concepto, $email_pagador);
		} catch (Exception $e) {
			echo($e);
		}

		$flow_url_pago = $this->getConfigValue('flow_url');
		echo '
		<html>
			<head></head>
			<body onload="sendData();">
				<form id="flowForm" method="post" action="'.$flow_url_pago.'">
					<input type="hidden" name="parameters" value="'.$flow_pack.'" />
				</form>
				<script type="text/javascript">
					function sendData() {
						var form = document.getElementById("flowForm");
						form.submit();
					}
				</script>
			</body>
		</html>
		';
	}

	public function indexAction()
	{
		Mage_Core_Controller_Varien_Action::_redirect('', array('_secure'=> false));
	}

	public function confirmAction()
	{
		Mage::log("[FlowPayment] Entering the confirm action...");

		$flowAPI = $this->getFlowAPI();
		$response = null;

		try {
			$flowAPI->read_confirm();

			$orderId = $flowAPI->getOrderNumber();
			$order = Mage::getModel('sales/order')->loadByIncrementId($orderId);
			$paymentState = $flowAPI->getStatus() == "EXITO";
			$response =  $flowAPI->build_response($paymentState);

			if($paymentState) {
				$this->payOrder($order, $orderId);
				$order->sendNewOrderEmail();
			} else {
	      		$this->cancelOrder($order);
			}

			$order->save();
			Mage::getSingleton('checkout/session')->unsQuoteId();
		} catch (Exception $e) {
			Mage::logException($e);
			$response = $flowAPI ->build_response(false);
		}

		echo $response;
	}

	public function successAction()
	{
		Mage::log("[FlowPayment] Entering the success action...");

		$flowAPI = $this->getFlowAPI();
		$flowAPI->read_confirm(false);

		$orderId = intval($flowAPI->getOrderNumber());
		$order = Mage::getModel('sales/order')->loadByIncrementId($orderId);
		$orderStatus = $order->getStatus();

		$successState = $this->getConfigValue("flow_order_status_success");

		//Getting the cart that belongs to the order
		$quote = Mage::getSingleton('sales/quote')->load($orderId, 'reserved_order_id');

		//If the order is not already paid
		if ($orderStatus != $successState) {
			Mage::log("[FlowPayment] Paying order in success action...");
			$this->payOrder($order, $flowAPI->getFlowNumber());
			$order->sendNewOrderEmail();
		}

		$session = Mage::getSingleton('checkout/session');

		if (!$session->getLastSuccessQuoteId()) {
			$session->setLastSuccessQuoteId($quote->getId());
		}

		if (!$session->getLastOrderId()) {
			$session->setLastOrderId($orderId);
		}

		if (!$session->getLastQuoteId()) {
			$session->setLastQuoteId($quote->getId());
		}

		$customer = Mage::getSingleton('customer/session');

		//Restoring the customer so magento doesn't log out
		if (!$customer->getId()) {
			$customerId = $order->getCustomerId();
			$customer->setId($customerId);
		}

		Mage_Core_Controller_Varien_Action::_redirect('checkout/onepage/success', array('_secure'=> false));
	}

	public function failAction()
	{
		Mage::log("[FlowPayment] Entering the fail action...");

		$flowAPI = $this->getFlowAPI();
		$flowAPI->read_confirm(false);

		$orderId = intval($flowAPI->getOrderNumber());
		$order = Mage::getModel('sales/order')->loadByIncrementId($orderId);
		$orderStatus = $order->getStatus();

		$failureState = $this->getConfigValue("order_status_fail");

		//if the order is not already canceled
		if ($orderStatus != $failureState) {
			Mage::log("[FlowPayment] Canceling order and recovering cart...");
			$this->cancelOrder($order);
		}

		$this->recoverCart($order);

		Mage_Core_Controller_Varien_Action::_redirect('checkout/onepage/failure', array('_secure'=> false));
	}

	private function getFlowAPI()
	{
		$flowAPI = new flowAPI();
		
		$flowAPI->setMedioPago($this->getConfigValue('medio_pago'));
		$flowAPI->setComercio($this->getConfigValue('id_comercio'));
		$flowAPI->setURLExito(Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_WEB).'index.php/flowpayment/checkout/success');
		$flowAPI->setURLFracaso(Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_WEB).'index.php/flowpayment/checkout/fail');
		$flowAPI->setURLConfirmacion(Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_WEB).'index.php/flowpayment/checkout/confirm');
		$flowAPI->setTipoIntegracion($this->getConfigValue('tipo_integracion'));
		$flowAPI->setKeyCommerce($this->getConfigValue('certificate'));
		$flowAPI->setKeysFolder(Mage::getBaseDir('base').'/var/uploads/flow');
		$flowAPI->setLogFolder(Mage::getBaseDir('base').'/var/log');
		return $flowAPI;
	}

	private function getConfigValue($key) 
	{
		return Mage::getStoreConfig('payment/flowpayment/'.$key, Mage::app()->getStore()); 
	}

	private function createAndPayInvoice($order, $flowOrderNumber)
	{
		//Creating the invoice and setting it as paid!
		$payment = $order->getPayment();
		$payment->setTransactionId($flowOrderNumber)
				->setPreparedMessage('order paid')
				->setParentTransactionId($flowOrderNumber)
				->setShouldCloseParentTransaction(true)
				->setIsTransactionClosed(0)
				->registerCaptureNotification($order->getGrandTotal());
    }

    private function acceptOrder($order)
    {
		$this->setOrderState($order, $this->getConfigValue('flow_order_status_success'), true, 'Payment successful');
    }

	private function cancelOrder($order)
	{
		Mage::log("[FlowPayment] Canceling order: ".$order->getId());

		$this->setOrderState($order, $this->getConfigValue('order_status_fail'), true, 'Payment unsuccesful');
		$order->save();
	}

	private function setOrderState($order, $state, $comment)
	{
        $order->setState($state, true, $comment);
    }

	private function recoverCart($order)
	{
        Mage::log('[FlowPayment] Recovering cart...');

		//Use convert_order to convert the order address object into a quote address object
		$convertObj      = Mage::getSingleton('sales/convert_order');
		$billing_address  = $convertObj->addressToQuoteAddress($order->getBillingAddress());
		$shipping_address = $convertObj->addressToQuoteAddress($order->getShippingAddress());
		
		//Get the session & cart variables
		$session = Mage::getSingleton('core/session');
		$cart = Mage::getSingleton('checkout/cart');
		
		//Retrieve individual items & populate the cart with each
		$items = $order->getAllVisibleItems();
		foreach ($items as $item) {
			try {
				$product = Mage::getModel('catalog/product')->load($item->getProductId());
				$options = $item->getProduct()->getTypeInstance(true)->getOrderOptions($item->getProduct());
				$info = $options['info_buyRequest'];
				$request = new Varien_Object();
				$request->setData($info);
				$cart->addProduct($product, $request);
			}
			catch (Exception $e) {
				Mage::getSingleton('core/session')->addError($e->getMessage());
			}
		}
		$cart->save();
		
		//Load the address into the checkout session
		try {
			$checkout = Mage::getSingleton('checkout/session')->getQuote();
			$checkout->setBillingAddress($billing_address);
			$checkout->setShippingAddress($shipping_address);
		}
		catch (Exception $e) {
			Mage::getSingleton('core/session')->addError($e->getMessage());
		}
		
		//Update cart after everything done
		$session->setCartWasUpdated(true);

    }

	private function payOrder($order, $orderNumber)
	{
		$this->createAndPayInvoice($order, $orderNumber);
		$this->acceptOrder($order);
		$order->save();
	}
}